program main

  use vtype_module

  use fi_module
  use cia_module

  implicit none


  real(dp)           , parameter :: Factor = 1.0d0
!!$  real(dp)           , parameter :: Factor = 6.0d0
!!$  real(dp)           , parameter :: Factor = 1.0d1

  character(extstr) :: CtlFN  = 'conv_cia2nc.conf'

  character(extstr) :: CIAFileName
  character(extstr) :: OutputFileName
  character(extstr*2) :: Src
  character(extstr  ) :: Ref


  ! Local variables
  !

  integer , parameter   :: MaxNmlArySize = 256


  type( CIAParam ) :: CIAP

  INTEGER, PARAMETER :: nS = 3000 !c-mm  Number of wavenumbers of data available
  INTEGER, PARAMETER :: nT = 9    !c-mm  Number of temperatures of data available

  REAL(8) :: wn_arr(nS)
  REAL(8) :: temp_arr(nT)
  REAL(8) :: dim_arr(nS,nT)

  integer              :: NWaveNum
  integer              :: NTemp
  REAL(8), allocatable :: w_WaveNum(:)
  REAL(8), allocatable :: t_Temp   (:)
  REAL(8), allocatable :: wt_Coef  (:,:)

  integer              :: NWaveNumEdge
  integer              :: NWaveNumBK
  REAL(8)              :: DelWaveNum
  REAL(8), allocatable :: w_WaveNumBK(:)
  REAL(8), allocatable :: wt_CoefBK  (:,:)
  REAL(8), allocatable :: wt_CoefLog (:,:)

  integer :: NDataLSF
  real(8) :: SumX
  real(8) :: SumY
  real(8) :: SumXSq
  real(8) :: SumXY
  real(8) :: CoefA
  real(8) :: CoefB

  integer            :: FileUnit
  integer            :: iostat

  integer            :: i
  integer            :: j
  integer            :: is
  integer            :: ie


  namelist /file_nml/ &
    & CIAFileName, &
    & OutputFileName, &
    & Src, &
    & Ref


  CIAFileName    = "../data_hitran/CIA/data/CO2-CO2_2011.cia"
  OutputFileName = "./out"


  ! Explanation

  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = file_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in file_nml is something wrong.'
  end if
  write( 6, nml = file_nml )
  !
  close( FileUnit )


  write( 6, * ) 'Input  File: ', trim( CIAFileName )
  write( 6, * ) 'Output File: ', trim( OutputFileName )


  call read_baranov_data(wn_arr,temp_arr,dim_arr)

!!$  do i = 1, nS
!!$    if ( 1200.0d0 <= wn_arr(i) ) exit
!!$  end do
!!$  is = i-1
!!$  do i = nS, 1, -1
!!$    if ( wn_arr(i) <= 1500.0d0 ) exit
!!$  end do
!!$  ie = i+1

  is = 1
  ie = nS

  NWaveNum = ie - is + 1
  NTemp    = nT
  allocate( w_WaveNum( NWaveNum ) )
  allocate( t_Temp   ( NTemp    ) )
  allocate( wt_Coef  ( NWaveNum, NTemp ) )
  do i = 1, NWaveNum
    w_WaveNum(i) = wn_arr(i-1+is)
  end do
  do j = 1, NTemp
    t_Temp(j) = temp_arr(j)
  end do
  do j = 1, NTemp
    do i = 1, NWaveNum
      wt_Coef(i,j) = dim_arr(i-1+is,j)
    end do
  end do


  ! Unit conversion
  !
  !   conversion from cm-1 to m-1
  w_WaveNum = w_WaveNum * 1.0d2
  !
  !   conversion from cm-1 amagat-2 to cm5 molecule-2
  wt_Coef = wt_Coef * 1.385d-39
  !   conversion from cm5 molecule-2 to m5 molecule-2
  wt_Coef = wt_Coef * 1.0d-10


  wt_Coef = wt_Coef * Factor


  do j = 1, NTemp
    do i = 1, NWaveNum
      write( 60+j, * ) w_WaveNum(i)*1.0d-2, wt_Coef(i,j)
    end do
  end do


  ! Extention
  !
  NWaveNumEdge = 500
  DelWaveNum   = 1.0d0 * 1.0d2
  NDataLSF     = 100


  NWaveNumBK = NWaveNum
  allocate( w_WaveNumBK( NWaveNumBK ) )
  allocate( wt_CoefBK  ( NWaveNumBK, NTemp ) )
  w_WaveNumBK = w_WaveNum
  wt_CoefBK   = wt_Coef
  !
  NWaveNum = NWaveNumEdge + NWaveNum + NWaveNumEdge
  deallocate( w_WaveNum )
  deallocate( wt_Coef   )
  allocate( w_WaveNum( NWaveNum ) )
  allocate( wt_Coef  ( NWaveNum, NTemp ) )
  !
  do i = NWaveNumEdge+1, NWaveNum-NWaveNumEdge
    w_WaveNum(i) = w_WaveNumBK(i-NWaveNumEdge)
  end do
  do j = 1, NTemp
    do i = NWaveNumEdge+1, NWaveNum-NWaveNumEdge
      wt_Coef(i,j) = wt_CoefBK(i-NWaveNumEdge,j)
    end do
  end do
  !
  do i = NWaveNumEdge, 1, -1
    w_WaveNum(i) = w_WaveNum(i+1) - DelWaveNum
  end do
  do i = NWaveNum-NWaveNumEdge+1, NWaveNum
    w_WaveNum(i) = w_WaveNum(i-1) + DelWaveNum
  end do
  !
  allocate( wt_CoefLog  ( NWaveNum, NTemp ) )
  do j = 1, NTemp
    do i = NWaveNumEdge+1, NWaveNum-NWaveNumEdge
      wt_CoefLog(i,j) = log( wt_Coef(i,j) )
    end do
  end do

  do j = 1, NTemp
    SumX   = 0.0d0
    SumY   = 0.0d0
    SumXSq = 0.0d0
    SumXY  = 0.0d0
    do i = NWaveNumEdge+1, NWaveNumEdge+NDataLSF
      SumX   = SumX   + w_WaveNum(i)
      SumY   = SumY   + wt_CoefLog(i,j)
      SumXSq = SumXSq + w_WaveNum(i)**2
      SumXY  = SumXY  + w_WaveNum(i) * wt_CoefLog(i,j)
    end do
    CoefA = ( SumXY * NDataLSF - SumX * SumY ) / ( SumXSq * NDataLSF - SumX**2 )
    CoefB = ( SumXSq * SumY - SumXY * SumX ) / ( SumXSq * NDataLSF - SumX**2 )
    do i = 1, NWaveNumEdge
      wt_CoefLog(i,j) = CoefA * w_WaveNum(i) + CoefB
      wt_Coef   (i,j) = exp( wt_CoefLog(i,j) )
    end do
    !
    SumX   = 0.0d0
    SumY   = 0.0d0
    SumXSq = 0.0d0
    SumXY  = 0.0d0
    do i = NWaveNum-NWaveNumEdge-NDataLSF+1, NWaveNum-NWaveNumEdge
      SumX   = SumX   + w_WaveNum(i)
      SumY   = SumY   + wt_CoefLog(i,j)
      SumXSq = SumXSq + w_WaveNum(i)**2
      SumXY  = SumXY  + w_WaveNum(i) * wt_CoefLog(i,j)
    end do
    CoefA = ( SumXY * NDataLSF - SumX * SumY ) / ( SumXSq * NDataLSF - SumX**2 )
    CoefB = ( SumXSq * SumY - SumXY * SumX ) / ( SumXSq * NDataLSF - SumX**2 )
    do i = NWaveNum-NWaveNumEdge+1, NWaveNum
      wt_CoefLog(i,j) = CoefA * w_WaveNum(i) + CoefB
      wt_Coef   (i,j) = exp( wt_CoefLog(i,j) )
    end do
  end do

  do j = 1, NTemp
    do i = 1, NWaveNum
      write( 80+j, * ) w_WaveNum(i)*1.0d-2, wt_Coef(i,j), wt_CoefLog(i,j)
    end do
  end do

  call CIAStoreDataInObj( &
    & 2, 2, &
    & NWaveNum, NTemp, w_WaveNum, t_Temp, wt_Coef, &
    & CIAP  &
    & )

!!$  call CIAReadFile( CIAFileName, CIAP )
  call CIAWriteFileNetcdf( OutputFileName, Src, Ref, CIAP )
  call CIAParamFinalize( CIAP )


end program main
