program main

  use vtype_module

  use fi_module
  use cia_module

  implicit none

  real(dp), parameter :: Factor = 1.0d0
!!$  real(dp), parameter :: Factor = 6.0d0
!!$  real(dp), parameter :: Factor = 1.0d1

  character(extstr) :: CtlFN  = 'conv_cia2nc.conf'

  character(extstr) :: CIAFileName
  character(extstr) :: OutputFileName
  character(extstr) :: Src
  character(extstr) :: Ref

  real(dp) :: WaveNumS
  real(dp) :: WaveNumE

  ! Local variables
  !

  integer , parameter   :: MaxNmlArySize = 256

  type( CIAParam ) :: CIAP
  type( CIAParam ) :: CIAPOut

  integer               :: NWaveNum
  integer               :: iWaveNumS
  integer               :: iWaveNumE
  integer               :: NTemp
  real(dp), allocatable :: w_WaveNum(:)
  real(dp), allocatable :: wt_Coef  (:,:)

  integer              :: NWaveNumEdge
  integer              :: NWaveNumBK
  REAL(8)              :: DelWaveNum
  REAL(8), allocatable :: w_WaveNumBK(:)
  REAL(8), allocatable :: wt_CoefBK  (:,:)
  REAL(8), allocatable :: wt_CoefLog (:,:)

  integer :: NDataLSF
  real(8) :: SumX
  real(8) :: SumY
  real(8) :: SumXSq
  real(8) :: SumXY
  real(8) :: CoefA
  real(8) :: CoefB

  integer            :: FileUnit
  integer            :: iostat

  integer            :: i
  integer            :: j
  integer            :: l


  namelist /wn_nml/ &
    & WaveNumS, &
    & WaveNumE

  namelist /file_nml/ &
    & CIAFileName, &
    & OutputFileName, &
    & Src, &
    & Ref


  WaveNumS = -1.0d0
  WaveNumE = -1.0d0

  CIAFileName    = "../data_hitran/CIA/data/CO2-CO2_2011.cia"
  OutputFileName = "./out"


  ! Explanation

  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = wn_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in wn_nml is something wrong.'
  end if
  write( 6, nml = wn_nml )
  !
  rewind( FileUnit )
  read( FileUnit, nml = file_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in file_nml is something wrong.'
  end if
  write( 6, nml = file_nml )
  !
  close( FileUnit )


  write( 6, * ) 'Input  File: ', trim( CIAFileName )
  write( 6, * ) 'Output File: ', trim( OutputFileName )

  call CIAReadHITRANCIAFile( CIAFileName, CIAP )

  if ( WaveNumS < 0.0d0 ) then
    iWaveNumS = 1
  else
    do l = 1, CIAP%NWaveNum
      if ( WaveNumS <= CIAP%w_WaveNum(l) ) then
        exit
      end if
    end do
    iWaveNumS = l
  end if
  if ( WaveNumE < 0.0d0 ) then
    iWaveNumE = CIAP%NWaveNum
  else
    do l = CIAP%NWaveNum, 1, -1
      if ( CIAP%w_WaveNum(l) <= WaveNumE ) then
        exit
      end if
    end do
    iWaveNumE = l
  end if

  NWaveNum = iWaveNumE - iWaveNumS + 1
  NTemp    = CIAP%NTemp
  allocate( w_WaveNum( NWaveNum ) )
  allocate( wt_Coef  ( NWaveNum, NTemp ) )
  do l = iWaveNumS, iWaveNumE
    w_WaveNum(l-iWaveNumS+1) = CIAP%w_WaveNum(l)
  end do
  do i = 1, NTemp
    do l = iWaveNumS, iWaveNumE
      wt_Coef(l-iWaveNumS+1,i) = CIAP%wt_Coef(l,i)
    end do
  end do



  ! Extention
  !
  NWaveNumEdge = 250
  DelWaveNum   = 1.0d0 * 1.0d2
  NDataLSF     = 10

  NWaveNumBK = NWaveNum
  allocate( w_WaveNumBK( NWaveNumBK ) )
  allocate( wt_CoefBK  ( NWaveNumBK, NTemp ) )
  w_WaveNumBK = w_WaveNum
  wt_CoefBK   = wt_Coef
  !
  NWaveNum = NWaveNum + NWaveNumEdge
  deallocate( w_WaveNum )
  deallocate( wt_Coef   )
  allocate( w_WaveNum( NWaveNum ) )
  allocate( wt_Coef  ( NWaveNum, NTemp ) )
  !
  do i = 1, NWaveNum-NWaveNumEdge
    w_WaveNum(i) = w_WaveNumBK(i)
  end do
  do j = 1, NTemp
    do i = 1, NWaveNum-NWaveNumEdge
      wt_Coef(i,j) = wt_CoefBK(i,j)
    end do
  end do
  !
  do i = NWaveNum-NWaveNumEdge+1, NWaveNum
    w_WaveNum(i) = w_WaveNum(i-1) + DelWaveNum
  end do
  !
  allocate( wt_CoefLog  ( NWaveNum, NTemp ) )
  do j = 1, NTemp
    do i = 1, NWaveNum-NWaveNumEdge
      wt_CoefLog(i,j) = log( wt_Coef(i,j) )
    end do
  end do

  do j = 1, NTemp
    SumX   = 0.0d0
    SumY   = 0.0d0
    SumXSq = 0.0d0
    SumXY  = 0.0d0
    do i = NWaveNum-NWaveNumEdge-NDataLSF+1, NWaveNum-NWaveNumEdge
      SumX   = SumX   + w_WaveNum(i)
      SumY   = SumY   + wt_CoefLog(i,j)
      SumXSq = SumXSq + w_WaveNum(i)**2
      SumXY  = SumXY  + w_WaveNum(i) * wt_CoefLog(i,j)
    end do
    CoefA = ( SumXY * NDataLSF - SumX * SumY ) / ( SumXSq * NDataLSF - SumX**2 )

    CoefB = ( SumXSq * SumY - SumXY * SumX ) / ( SumXSq * NDataLSF - SumX**2 )
    do i = NWaveNum-NWaveNumEdge+1, NWaveNum
      wt_CoefLog(i,j) = CoefA * w_WaveNum(i) + CoefB
      wt_Coef   (i,j) = exp( wt_CoefLog(i,j) )
    end do
  end do


  wt_Coef = wt_Coef * Factor

  do j = 1, NTemp
    do i = 1, NWaveNum
      write( 80+j, * ) w_WaveNum(i)*1.0d-2, wt_Coef(i,j), wt_CoefLog(i,j)
    end do
  end do



  call CIAStoreDataInObj( &
    & CIAP%IMol, CIAP%IMolOpp, &
    & NWaveNum, NTemp, w_WaveNum, CIAP%t_Temp, wt_Coef, &
    & CIAPOut  &
    & )
  deallocate( w_WaveNum )
  deallocate( wt_Coef   )

  call CIAWriteFileNetcdf( OutputFileName, Src, Ref, CIAPOut )

  call CIAParamFinalize( CIAP    )
  call CIAParamFinalize( CIAPOut )


end program main
