program main

  use vtype_module
  use io_wnlist, only :   &
    & IOWNListOutput

  use fi_module

  implicit none

  character(128) :: ctlfn = 'mkwnlist.conf'

  character(128) :: OutNcFN
  character(128) :: OutASCIIFN


  integer             , parameter   :: NChar = stdstr

  integer, parameter :: MaxNmlArySize = 256
  real(dp)           :: WaveNumLim(MaxNmlArySize)
  real(dp)           :: DelWaveNum(MaxNmlArySize)

  integer               :: NWaveNumReg
  real(dp), allocatable :: a_WaveNumS  (:)
  real(dp), allocatable :: a_WaveNumE  (:)
  real(dp), allocatable :: a_DelWaveNum(:)
  integer , allocatable :: a_NWaveNum  (:)

  integer               :: NWaveNum
  real(dp), allocatable :: w_WaveNum   (:)

  logical :: FlagNC
  integer :: FileUnit
  integer :: iostat

  integer :: iWaveNum
  integer :: i
  integer :: n


  namelist /wavenum_nml/ &
    & NWaveNumReg, WaveNumLim, DelWaveNum

  namelist /file_nml/ &
    & OutNcFN, OutASCIIFN

  WaveNumLim(:) = -1.0d100
  DelWaveNum(:) = -1.0d100

  OutNcFN     = ''
  OutASCIIFN  = ''


  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = wavenum_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist, wavenum_nml, is something wrong.'
  end if
  write( 6, nml = wavenum_nml )
  !
  rewind( FileUnit )
  read( FileUnit, nml = file_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist, file_nml, is something wrong.'
  end if
  write( 6, nml = file_nml )
  !
  close( FileUnit )


  ! check
  if ( WaveNumLim(1) < 0.0d0 ) then
    write( 6, * ) 'ERROR : WaveNumLim(',1,') has to be >= 0.'
    stop
  end if
  do n = 1, NWaveNumReg
    if ( WaveNumLim(n+1) <= 0.0d0 ) then
      write( 6, * ) 'ERROR : WaveNumLim(',n+1,') has to be > WaveNumLim(',n,').'
      write( 6, * ) '        WaveNumLim(',n+1,') =', WaveNumLim(n+1)
      write( 6, * ) '        WaveNumLim(',n  ,') =', WaveNumLim(n  )
      stop
    end if
    if ( DelWaveNum(n  ) <= 0.0d0 ) then
      write( 6, * ) 'ERROR : DelWaveNum(',n  ,') <= 0 though WaveNumLim(',n+1,') > 0,'
      write( 6, * ) '        DelWaveNum(',n  ,') =', DelWaveNum(n  )
      write( 6, * ) '        WaveNumLim(',n+1,') =', WaveNumLim(n+1)
      stop
    end if
  end do
  allocate( a_WaveNumS  (NWaveNumReg) )
  allocate( a_WaveNumE  (NWaveNumReg) )
  allocate( a_DelWaveNum(NWaveNumReg) )
  allocate( a_NWaveNum  (NWaveNumReg) )
  a_WaveNumS   = WaveNumLim(1:NWaveNumReg  )
  a_WaveNumE   = WaveNumLim(2:NWaveNumReg+1)
  a_DelWaveNum = DelWaveNum(1:NWaveNumReg  )
  a_NWaveNum   = int( ( a_WaveNumE - a_WaveNumS ) / a_DelWaveNum )

  NWaveNum = 0
  do n = 1, NWaveNumReg
    NWaveNum = NWaveNum + a_NWaveNum(n)
  end do

  allocate( w_WaveNum(NWaveNum) )
  iWaveNum = 1
  do n = 1, NWaveNumReg
    do i = 1, a_NWaveNum(n)
      w_WaveNum(iWaveNum) = &
        & a_WaveNumS(n) + a_DelWaveNum(n) / 2.0d0 + a_DelWaveNum(n) * dble(i-1)
      iWaveNum = iWaveNum + 1
    end do
  end do

  ! check
  do iWaveNum = 1+1, NWaveNum
    if ( w_WaveNum(iWaveNum-1) > w_WaveNum(iWaveNum) ) then
      write( 6, * ) 'ERROR : w_WaveNum is not in ascending order.'
      write( 6, * ) '      : ', w_WaveNum
      stop
    end if
  end do


  write( 6, * )
  write( 6, * ) 'Number of regions : ', NWaveNumReg
  do n = 1, NWaveNumReg
    write( 6, * ) 'region #      : ', n
    write( 6, * ) '  range       : ', a_WaveNumS(n), '-', a_WaveNumE(n), ' (m-1)'
    write( 6, * ) '  interval    : ', a_DelWaveNum(n), ' (m-1)'
    write( 6, * ) '  # of points : ', a_NWaveNum(n)
  end do
  write( 6, * ) 'Total number of wavenumber point : ', NWaveNum


  ! output
  FlagNC = .true.
  call IOWNListOutput(                    &
    & FlagNC, OutNcFN   , NWaveNum, w_WaveNum  &
    & )
  FlagNC = .false.
  call IOWNListOutput(                    &
    & FlagNC, OutASCIIFN, NWaveNum, w_WaveNum  &
    & )


  write( 6, * ) 'Finish'

end program main
