program main

  use vtype_module

#ifdef MPI
  use mpi
#endif

  use fi_module

  use io_profile, only : &
    & IOProfileInqDimLen, &
    & IOProfileRead
  use io_wnlist, only : &
    & IOWNListInqDimLen, &
    & IOWNListRead

  use ac_calc, only : ACCalc

  use ac_calc_utils, only : &
    & ACCalcUtilsModifyWavenumberRange1, &
    & ACCalcUtilsModifyWavenumberRange2, &
    & ACCalcUtilsModifyWavenumberRange3


  implicit none


  character(extstr) :: CtlFN  = 'calc_ac.conf'
  character(extstr) :: CtlFN2 = 'calc_ac_file.conf'
  character(extstr) :: CtlFN3 = 'krange.conf'

  integer           :: iostat

  logical , allocatable :: m_FlagCalcLine(:)
  logical , allocatable :: m_FlagCalcCont(:)

  integer, parameter :: MaxNmlArySize = 64

  logical                            :: FlagCalcAllLevels

  logical                            :: FlagNetCDF

  character(len=extstr)              :: HITFileName(MaxNmlArySize)
  integer                            :: NHIT
  character(len=extstr), allocatable :: a_HITFileName(:)
  character(len=extstr)              :: OutputFileName
#ifdef MPI
  character(len=extstr)              :: OutputFileNameTmp
#endif
  character(len=extstr), parameter   :: CVersion = "v1"

  integer                       :: NLev
  integer                       :: NLevOut

  !
  ! NMolNum : number of radiatively active species
  !
  integer                        :: NMolNum


  integer(i4b)              :: FileUnit



  logical              , parameter :: FlagNC = .true.
  character(len=extstr)            :: WaveNumFileName

  character(len=extstr)     :: AtmProfFileName
  character(len=extstr)     :: PressName
  character(len=extstr)     :: MolNumName
  character(len=extstr)     :: TempName
  character(len=extstr)     :: SurfTempName
  character(len=extstr)     :: VMRName

  integer     , allocatable :: m_MolNum( : )
  real(dp)    , allocatable :: r_Press(:)
  real(dp)    , allocatable :: r_Temp(:)
  real(dp)                  :: SurfTemp
  real(dp)    , allocatable :: rm_VMR(:,:)


  real(dp)    , allocatable :: z_Press(:)
  real(dp)    , allocatable :: z_Temp(:)
  real(dp)    , allocatable :: zm_VMR(:,:)
  real(dp)    , allocatable :: zm_VMRShrink(:,:)

  integer                   :: NWaveNum
  real(dp)    , allocatable :: w_WaveNum(:)


  logical                            :: FlagCalcLine(MaxNmlArySize)
  logical                            :: FlagCalcCont(MaxNmlArySize)

  real(dp)                           :: LineCutOffWaveNum(MaxNmlArySize)
  real(dp), allocatable              :: m_LineCutOffWaveNum(:)

  real(dp)                           :: AirBroadHalfWidFac(MaxNmlArySize)
  real(dp), allocatable              :: m_AirBroadHalfWidFac(:)

  character(len=extstr)              :: CIAFileName(MaxNmlArySize)
  integer                            :: NCIA
  character(len=extstr), allocatable :: a_CIAFileName(:)

  character(len=extstr)              :: XSCFileName(MaxNmlArySize)
  integer                            :: NXSC
  character(len=extstr), allocatable :: a_XSCFileName(:)


  integer :: kMin
  integer :: kMax

  integer :: ks
  integer :: ke

  integer :: k
  integer :: m
  integer :: n


  integer :: NProcs
  integer :: MyRank
#ifdef MPI
  integer :: iRank
  integer :: IErr
#endif

  integer, parameter :: FileNumS = 0


  namelist /gas_nml/ &
    & FlagCalcLine, FlagCalcCont, &
    & FlagNetCDF, &
    & HITFileName, CIAFileName, XSCFileName, &
    & LineCutOffWaveNum, &
    & AirBroadHalfWidFac
  namelist /file_nml/ &
    & WaveNumFileName, &
    & AtmProfFileName, &
    & OutputFileName, &
    & PressName, MolNumName, TempName, SurfTempName, VMRName, &
    & FlagCalcAllLevels
  namelist /krange_nml/ &
    & kMin, kMax

#ifdef MPI
  call mpi_init( IErr )
  call mpi_comm_size( mpi_comm_world, NProcs, IErr )
  call mpi_comm_rank( mpi_comm_world, MyRank, IErr )
#else
  NProcs = 1
  MyRank = 0
#endif



  FlagCalcLine(:) = .true.
  FlagCalcCont(:) = .true.

  FlagNetCDF = .true.
  HITFileName(:) = ""
  !
  CIAFileName(:)  = ""
!  CIAFileName(1)  = "../data/CIA/NC_data/02-02_cia.nc"
  !
  XSCFileName(:)  = ""
  !
  OutputFileName = "./out/ac.nc"

  LineCutOffWaveNum(:) = 25.0d2

  AirBroadHalfWidFac = 1.0d0

  !======================
  WaveNumFileName   = "./wnlist.nc"
  AtmProfFileName   = "./atmprof.nc"
  PressName         = "Press"
  MolNumName        = "MolNum"
  TempName          = "Temp"
  SurfTempName      = "Ts"
  VMRName           = "VMR"
  FlagCalcAllLevels = .false.
  !======================



  ! Read a namelist file

  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = gas_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in gas_nml is something wrong.'
  end if
  write( 6, nml = gas_nml )
  !
  close( FileUnit )

  call fi_open( CtlFN2, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = file_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in file_nml is something wrong.'
  end if
  write( 6, nml = file_nml )
  !
  close( FileUnit )

  call fi_findfu( CtlFN3, "read", iostat, FileUnit )
  if( iostat == 0 ) then
    open( FileUnit, file = CtlFN3, status = "unknown", iostat = iostat )
    !
    rewind( FileUnit )
    read( FileUnit, nml = krange_nml, iostat = iostat )
    if ( iostat > 0 ) then
      stop 'Namelist in krange_nml is something wrong.'
    end if
    write( 6, nml = krange_nml )
    !
    close( FileUnit )

    if ( kMin > kMax ) then
      write( 6, * ) 'Error : kMin has to be less than or equal to kMax.'
      write( 6, * ) kMin, kMax
      stop
    end if
  else
    write( 6, * ) 'Cannot open ', trim( CtlFN3 )
    kMin = -999
    kMax = -999
  end if


  call IOProfileInqDimLen(        &
    & AtmProfFileName, PressName, &
    & NLev &
    & )
  call IOProfileInqDimLen(         &
    & AtmProfFileName, MolNumName, &
    & NMolNum &
    & )
  allocate( m_MolNum( NMolNum ) )
  allocate( r_Press ( NLev    ) )
  allocate( r_Temp  ( NLev    ) )
  allocate( rm_VMR  ( NLev, NMolNum ) )
  call IOProfileRead(      &
    & AtmProfFileName, PressName, MolNumName, TempName, SurfTempName, VMRName, &
    & NLev, NMolNum, &
    & r_Press, m_MolNum, r_Temp, SurfTemp, rm_VMR     &
    & )
  ! Output for check
  write( 6, * ) '===================='
  write( 6, * ) 'Check atmospheric profile input from the specified file'
  write( 6, * ) 'Molecular number : ', m_MolNum(1:NMolNum)
  write( 6, * ) 'Surface temperature : ', SurfTemp
  write( 6, * ) 'Level index, Pressure, Temperature, Volume mixing ratios : '
  do k = 1, NLev
    write( 6, * ) k, r_Press(k), r_Temp(k), ( rm_VMR(k,n), n=1,NMolNum )
  end do
  write( 6, * ) '===================='

  if ( .not. FlagCalcAllLevels ) then
    NLevOut = NLev-1
    NLevOut = NLev-1
  else
    NLevOut = NLev
    NLevOut = NLev
  end if

  allocate( z_Press ( NLevOut   ) )
  allocate( z_Temp  ( NLevOut   ) )
  allocate( zm_VMR  ( NLevOut, NMolNum ) )

  if ( .not. FlagCalcAllLevels ) then
    do k = 1, NLevOut
      z_Press ( k ) = ( r_Press(k) + r_Press(k+1) ) / 2.0d0
      z_Temp  ( k ) = ( r_Temp (k) + r_Temp (k+1) ) / 2.0d0
    end do
    do m = 1, NMolNum
      do k = 1, NLevOut
        zm_VMR  ( k, m ) = ( rm_VMR(k,m) + rm_VMR(k+1,m) ) / 2.0d0
      end do
    end do
  else
    z_Press = r_Press
    z_Temp  = r_Temp
    zm_VMR  = rm_VMR
  end if


  allocate( m_FlagCalcLine(NMolNum) )
  allocate( m_FlagCalcCont(NMolNum) )
  m_FlagCalcLine = FlagCalcLine(1:NMolNum)
  m_FlagCalcCont = FlagCalcCont(1:NMolNum)
  write( 6, * ) 'Flags for calculation of lines and continuum:'
  do m = 1, NMolNum
    write( 6, '(a,l,a,l,a,i2)' ) &
      & '  Line = ', m_FlagCalcLine(m), &
      & ', Continuum = ', m_FlagCalcCont(m), &
      & ' for molecule ', m_MolNum(m)
  end do

  allocate( m_LineCutOffWaveNum(NMolNum) )
  m_LineCutOffWaveNum = LineCutOffWaveNum(1:NMolNum)
  write( 6, * ) 'Line cut off wavenumbers:'
  do m = 1, NMolNum
    write( 6, '(a,i4,a,i2)' ) &
      & '  Cut-off at ', int( m_LineCutOffWaveNum(m)*1d-2 ), &
      & ' cm-1 for molecule ', m_MolNum(m)
  end do

  allocate( m_AirBroadHalfWidFac(NMolNum) )
  m_AirBroadHalfWidFac = AirBroadHalfWidFac(1:NMolNum)
  write( 6, * ) 'Air-broadening half width factors:'
  do m = 1, NMolNum
    write( 6, '(a,f4.2,a,i2)' ) &
      & '  Air-broadening half width is multiplied by ', m_AirBroadHalfWidFac(m), &
      & ' for molecule ', m_MolNum(m)
  end do


  NHIT = 0
  do n = 1, MaxNmlArySize
    if ( len_trim( HITFileName(n) ) == 0 ) exit
  end do
  NHIT = n - 1
  allocate( a_HITFileName(NHIT) )
  do n = 1, NHIT
    a_HITFileName(n) = HITFileName(n)
  end do
  ! check
  if ( NHIT /= NMolNum ) then
    write( 6, * ) 'Number of HITRAN data files specified in gas_nml does not match to number of species in file, ', &
      trim( AtmProfFileName ), '.'
    write( 6, * ) 'Number of HITRAN data files: ', NHIT
    write( 6, * ) 'Number of species          : ', NMolNum
    stop
  end if

  NCIA = 0
  do n = 1, MaxNmlArySize
    if ( len_trim( CIAFileName(n) ) == 0 ) exit
  end do
  NCIA = n - 1
  allocate( a_CIAFileName(NCIA) )
  do n = 1, NCIA
    a_CIAFileName(n) = CIAFileName(n)
  end do

  NXSC = 0
  do n = 1, MaxNmlArySize
    if ( len_trim( XSCFileName(n) ) == 0 ) exit
  end do
  NXSC = n - 1
  allocate( a_XSCFileName(NXSC) )
  do n = 1, NXSC
    a_XSCFileName(n) = XSCFileName(n)
  end do


  ! read wavenumber
  call IOWNListInqDimLen( &
    & FlagNC, WaveNumFileName,    &
    & NWaveNum            &
    & )
  allocate( w_WaveNum( NWaveNum ) )
  call IOWNListRead(             &
    & FlagNC, WaveNumFileName, NWaveNum, &
    & w_WaveNum                  &
    & )


#ifdef MPI

#ifdef MPI_WN_DIVIDE

  write( 6, * ) 'Merge wavenumbers'
  !
  ! MPI for wavenumbers
  !
!  call ACCalcUtilsModifyWavenumberRange1( &
!    & NProcs, MyRank, &
!    & DelWaveNum, &
!    & WaveNumS, WaveNumE &
!    & )
!!$  call ACCalcUtilsModifyWavenumberRange2( &
!!$    & FlagNetCDF, NHIT, a_HITFileName, &
!!$    & NProcs, MyRank, &
!!$    & NMolNum, m_MolNum, &
!!$    & DelWaveNum, &
!!$    & WaveNumS, WaveNumE &
!!$    & )
  call ACCalcUtilsModifyWavenumberRange3( &
    & FlagNetCDF, NHIT, a_HITFileName, &
    & NProcs, MyRank, &
    & NMolNum, m_MolNum, m_LineCutOffWaveNum, &
    & DelWaveNum, &
    & WaveNumS, WaveNumE &
    & )

  write( OutputFileNameTmp, '(a,a,i6.6,a)' ) &
    & OutputFileName(1:len_trim(OutputFileName)-3), &
    & "_", &
    & FileNumS + MyRank, &
    & ".nc"
  OutputFileName = OutputFileNameTmp
  if ( MyRank == 0 ) then
    write( 6, * ) "# Wavenumber range is divided by MPI."
  end if
  write( 6, * ) "#", MyRank, ":", WaveNumS, WaveNumE, trim( OutputFileName )

  if ( ( kMin == -999 ) .and. ( kMax == -999 ) ) then
    ks = 1
    ke = NLevOut
  else
    ks = kMin
    ke = kMax
  end if

#else

  write( 6, * ) 'Merge vertical domain'
  !
  ! MPI for vertical levels
  !
  if ( NLevOut < NProcs ) then
    write( 6, * ) 'Number of processes is too large: ', NLevOut, NProcs
    stop
  end if
  !
  ke = 0
  do iRank = 0, MyRank
    ks = ke + 1
    ke = ks + NLevOut / NProcs - 1
    if ( iRank < mod( NLevOut, NProcs ) ) then
      ke = ke + 1
    end if
  end do
  ! check
  if ( ke > NLevOut ) then
    write( 6, * ) 'Unexpected number of maximum of loop: ', ke, NLevOut
  end if

  write( OutputFileNameTmp, '(a,a,i6.6,a)' ) &
    & OutputFileName(1:len_trim(OutputFileName)-3), &
    & "_", &
    & FileNumS + MyRank, &
    & ".nc"
  OutputFileName = OutputFileNameTmp
  if ( MyRank == 0 ) then
    write( 6, * ) "# Vertical domain is divided by MPI."
  end if
  write( 6, * ) "#", MyRank, ":", ks, ke, trim( OutputFileName )

#endif

#else
  ks = 1
  ke = NLevOut
#endif

  allocate( zm_VMRShrink( ks:ke, NMolNum ) )

  do m = 1, NMolNum
    do k = ks, ke
      zm_VMRShrink(k,m) = zm_VMR(k,m)
    end do
  end do


  call ACCalc( &
    & NWaveNum, w_WaveNum, &
    & FlagNetCDF, NHIT, a_HITFileName, &
    & NCIA, a_CIAFileName, &
    & NXSC, a_XSCFileName, &
    & OutputFileName, CVersion, &
!!$    & NLev, NMolNum, r_Press, m_MolNum, r_Temp, rm_VMR, &
!!$    & NLev-1, NMolNum, z_Press, m_MolNum, z_Temp, zm_VMR, &
    & ke-ks+1, NMolNum, z_Press(ks:ke), m_MolNum, z_Temp(ks:ke), zm_VMRShrink(ks:ke,:), &
    & m_FlagCalcLine, m_FlagCalcCont, &
    & m_LineCutOffWaveNum, m_AirBroadHalfWidFac &
    & )

  deallocate( w_WaveNum )

  deallocate( z_Press  )
  deallocate( z_Temp   )
  deallocate( zm_VMR   )

  deallocate( zm_VMRShrink )

  deallocate( a_HITFileName )
  deallocate( a_CIAFileName )
  deallocate( a_XSCFileName )
  deallocate( m_MolNum )
  deallocate( r_Press  )
  deallocate( r_Temp   )
  deallocate( rm_VMR   )
  deallocate( m_LineCutOffWaveNum )
  deallocate( m_FlagCalcLine )
  deallocate( m_FlagCalcCont )

  write( 6, * ) 'Finish'


#ifdef MPI
  call mpi_finalize( IErr )
#endif

end program main
