program main

  use vtype_module
  use fi_module
  use io_wnlist, only : &
    & IOWNListInqDimLen, &
    & IOWNListRead
  use particle_ec_calc, only : ParticleECCalc

#ifdef MPI
  use mpi

  use particle_ec_calc_utils, only : ParticleECCalcUtilsModifyWavenumberRange
#endif


  implicit none


  character(extstr) :: CtlFN  = 'calc_optprop.conf'
  character(extstr) :: CtlFN2 = 'calc_optprop_file.conf'

  integer           :: iostat

  logical              , parameter :: FlagNC = .true.
  character(len=extstr)            :: WaveNumFileName
  character(len=extstr)            :: OutputFileName
#ifdef MPI
  character(len=extstr)            :: OutputFileNameTmp
#endif

  integer, parameter :: MaxNmlArySize = 32

  integer            :: NMolNum
  integer            :: MolNum(MaxNmlArySize)

  integer            :: MolNumNonRadAct

  integer                   :: NWaveNum
  real(dp)    , allocatable :: w_WaveNum(:)

  logical            :: FlagReadQExtSSAAFFromFile (MaxNmlArySize)
  logical            :: FlagReadQExtSSAAFFromParam(MaxNmlArySize)

  integer            :: NPtcl
  character(stdstr)  :: PtclName        (MaxNmlArySize)
  character(stdstr)  :: PtclSDF         (MaxNmlArySize)
  character(extstr)  :: PtclDataFN      (MaxNmlArySize)
  integer            :: PtclNEffRadius  (MaxNmlArySize)
  real(dp)           :: PtclEffRadiusS  (MaxNmlArySize)
  real(dp)           :: PtclEffRadiusE  (MaxNmlArySize)
  real(dp)           :: PtclStdDev      (MaxNmlArySize)
  integer            :: NRadiusInteg    (MaxNmlArySize)

  integer          , allocatable :: m_MolNum(:)

  logical          , allocatable :: a_FlagReadQExtSSAAFFromFile (:)
  logical          , allocatable :: a_FlagReadQExtSSAAFFromParam(:)

  ! a_ParticleName       : Particle name
  ! a_ParticleSDF        : Particle size distribution function
  ! a_ParticleDataFN     : Data filename for optical properties
  ! a_ParticleNEffRadius : Number of effective radii
  ! a_ParticleEffRadiusS : Start effective radius in a table
  ! a_ParticleEffRadiusE : End   effective radius in a table
  ! a_ParticleStdDev     : Standard deviation of size distribution
  !                      : Valid only for lognormal
  ! a_NRadiusInteg       : Number of points for integration over radius
  ! a_RadiusIntegS       : Start radius for integration over radius
  ! a_RadiusIntegE       : End   radius for integration over radius

  character(stdstr), allocatable :: a_PtclName  (:)
  character(stdstr), allocatable :: a_PtclSDF   (:)
  character(extstr), allocatable :: a_PtclDataFN(:)

  integer          , allocatable :: a_PtclNEffRadius(:)
  real(dp)         , allocatable :: a_PtclEffRadiusS(:)
  real(dp)         , allocatable :: a_PtclEffRadiusE(:)

  real(dp)         , allocatable :: a_PtclStdDev      (:)
  integer          , allocatable :: a_NRadiusInteg    (:)

  integer(i4b)              :: FileUnit

  integer(i4b)              :: m
  integer(i4b)              :: n

  integer :: NProcs
  integer :: MyRank
#ifdef MPI
  integer :: IErr
#endif

  integer, parameter :: FileNumS = 0


  ! PtclName         : Particle name
  ! PtclSDF          : Particle size distribution function
  ! PtclDataFN       : Data filename for optical properties
  ! PtclNEffRadius   : Number of effective radii
  ! PtclEffRadiusS   : Start effective radius in a table
  ! PtclEffRadiusE   : End   effective radius in a table
  ! PtclStdDev       : Standard deviation of size distribution
  !                  : Valid only for lognormal
  ! NRadiusInteg     : Number of points for radius integration

  namelist /ptcl_nml/ &
    & FlagReadQExtSSAAFFromFile, &
    & FlagReadQExtSSAAFFromParam, &
    & MolNumNonRadAct, &
    & NMolNum, MolNum, &
    & NPtcl, PtclName, PtclSDF, PtclDataFN, &
    & PtclNEffRadius, PtclEffRadiusS, PtclEffRadiusE, &
    & PtclStdDev, &
    & NRadiusInteg

  namelist /file_nml/ &
    & WaveNumFileName, &
    & OutputFileName


#ifdef MPI
  call mpi_init( IErr )
  call mpi_comm_size( mpi_comm_world, NProcs, IErr )
  call mpi_comm_rank( mpi_comm_world, MyRank, IErr )
#else
  NProcs = 1
  MyRank = 0
#endif


  WaveNumFileName = "./wnlist.nc"
  OutputFileName  = "./out/ac.nc"

  FlagReadQExtSSAAFFromFile (:) = .false.
  FlagReadQExtSSAAFFromParam(:) = .false.

  MolNumNonRadAct = 0

  NMolNum   = 1
  MolNum    = -1
  MolNum(1) = 1
!!$      WaveNumS =    0.0d0 * 1.0d2
!!$      WaveNumE = 3000.0d0 * 1.0d2

  NPtcl         = 0
  PtclName  (:) = ""
  PtclSDF   (:) = ""
  PtclDataFN(:) = ""

  PtclNEffRadius(:) = 0
  PtclEffRadiusS(:) = -1.0d100
  PtclEffRadiusE(:) = -1.0d100

  PtclStdDev      (:) = -1.0d100
  NRadiusInteg    (:) = -1


  ! Read a namelist file

  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = ptcl_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in ptcl_nml is something wrong.'
  end if
  write( 6, nml = ptcl_nml )
  !
  close( FileUnit )

  call fi_open( CtlFN2, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = file_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in file_nml is something wrong.'
  end if
  write( 6, nml = file_nml )
  !
  close( FileUnit )


  allocate( m_MolNum         (NMolNum  ) )
  allocate( a_FlagReadQExtSSAAFFromFile (NPtcl) )
  allocate( a_FlagReadQExtSSAAFFromParam(NPtcl) )
  allocate( a_PtclName        (NPtcl) )
  allocate( a_PtclSDF         (NPtcl) )
  allocate( a_PtclDataFN      (NPtcl) )
  allocate( a_PtclNEffRadius  (NPtcl) )
  allocate( a_PtclEffRadiusS  (NPtcl) )
  allocate( a_PtclEffRadiusE  (NPtcl) )
  allocate( a_PtclStdDev      (NPtcl) )
  allocate( a_NRadiusInteg    (NPtcl) )

  m_MolNum = MolNum(1:NMolNum)

  a_FlagReadQExtSSAAFFromFile  = FlagReadQExtSSAAFFromFile (1:NPtcl)
  a_FlagReadQExtSSAAFFromParam = FlagReadQExtSSAAFFromParam(1:NPtcl)

  a_PtclName       = PtclName      (1:NPtcl)
  a_PtclSDF        = PtclSDF       (1:NPtcl)
  a_PtclDataFN     = PtclDataFN    (1:NPtcl)
  a_PtclNEffRadius = PtclNEffRadius(1:NPtcl)
  a_PtclEffRadiusS = PtclEffRadiusS(1:NPtcl)
  a_PtclEffRadiusE = PtclEffRadiusE(1:NPtcl)

  a_PtclStdDev       = PtclStdDev      (1:NPtcl)
  a_NRadiusInteg     = NRadiusInteg    (1:NPtcl)

  do m = 1, NMolNum
    if ( m_MolNum(m) <= 0 ) then
      write( 6, * ) 'Molecular number is inappropriate: ', m_MolNum
      stop
    end if
  end do
  do n = 1, NPtcl
    if ( a_PtclNEffRadius(n) <= 0 ) then
      write( 6, * ) 'Number of radius has to be positive.'
      stop
    end if
  end do
  do n = 1, NPtcl
    if ( ( .not. a_FlagReadQExtSSAAFFromFile(n)  ) .and. &
         ( .not. a_FlagReadQExtSSAAFFromParam(n) ) ) then

      if ( a_NRadiusInteg(n) <= 0 ) then
        write( 6, * ) 'Number of points for radius integration has to be positive.'
        stop
      end if
    end if
  end do


  ! read wavenumber
  call IOWNListInqDimLen(         &
    & FlagNC, WaveNumFileName,    &
    & NWaveNum                    &
    & )
  allocate( w_WaveNum( NWaveNum ) )
  call IOWNListRead(                     &
    & FlagNC, WaveNumFileName, NWaveNum, &
    & w_WaveNum                          &
    & )


#ifdef MPI
  write( 6, * ) 'Merge wavenumbers'
  !
  ! MPI for wavenumbers
  !
  call ParticleECCalcUtilsModifyWavenumberRange( &
    & NProcs, MyRank, &
    & DelWaveNum, &
    & WaveNumS, WaveNumE &
    & )
  if ( NProcs > 1 ) then
    write( OutputFileNameTmp, '(a,a,i6.6,a)' ) &
      & OutputFileName(1:len_trim(OutputFileName)-3), &
      & "_", &
      & FileNumS + MyRank, &
      & ".nc"
    OutputFileName = OutputFileNameTmp
  end if
  if ( MyRank == 0 ) then
    write( 6, * ) "# Wavenumber range is divided by MPI."
  end if
  write( 6, * ) "#", MyRank, ":", WaveNumS, WaveNumE, trim( OutputFileName )
#endif


  call ParticleECCalc( &
    & NWaveNum, w_WaveNum, &
    & MolNumNonRadAct, &
    & NMolNum, m_MolNum, &
    & NPtcl, &
    & a_FlagReadQExtSSAAFFromFile, &
    & a_FlagReadQExtSSAAFFromParam, &
    & a_PtclName, a_PtclSDF, a_PtclDataFN, &
    & a_PtclNEffRadius, a_PtclEffRadiusS, a_PtclEffRadiusE, &
    & a_PtclStdDev, &
    & a_NRadiusInteg, &
    & OutputFileName &
    & )

  deallocate( a_FlagReadQExtSSAAFFromFile  )
  deallocate( a_FlagReadQExtSSAAFFromParam )
  deallocate( a_PtclName         )
  deallocate( a_PtclSDF          )
  deallocate( a_PtclDataFN       )
  deallocate( a_PtclNEffRadius   )
  deallocate( a_PtclEffRadiusS   )
  deallocate( a_PtclEffRadiusE   )
  deallocate( a_PtclStdDev       )

  write( 6, * ) 'Finish'

#ifdef MPI
  call mpi_finalize( IErr )
#endif


end program main

