program main

  use vtype_module
  use fi_module

  use h2o_water, only : &
    & H2OWaterInit, &
    & H2OWaterFinalize, &
    & H2OWaterCalcParam
  use h2o_ice, only : &
    & H2OIceInit, &
    & H2OIceFinalize, &
    & H2OIceCalcParam

  implicit none

  real(dp) :: PartSize
  real(dp) :: Water_QExt
  real(dp) :: Water_SSA
  real(dp) :: Water_AF
  real(dp) :: Ice_QExt
  real(dp) :: Ice_SSA
  real(dp) :: Ice_AF

  integer :: iWavelength
  real(dp) :: Wavelength

  real(dp) :: RhoWater = 1.0d3
  real(dp) :: RhoIce   = 0.9167d3

  character(stdstr) :: Dir


  Dir = "data"

  PartSize = 2.5d-6
!!$  PartSize =  6.0d-6
  PartSize = 10.0d-6
!!$  PartSize = 50.0d-6

  call H2OWaterInit( Dir )
  call H2OIceInit  ( Dir )

  do iWavelength = 1, 1000
    Wavelength = 0.1d-6 * iWavelength

    call H2OWaterCalcParam( &
      & PartSize, Wavelength, &
      & Water_QExt, Water_SSA, Water_AF &
      & )
    call H2OIceCalcParam( &
      & PartSize, Wavelength, &
      & Ice_QExt, Ice_SSA, Ice_AF &
      & )

    write( 6, * ) Wavelength, Water_QExt, Water_SSA, Water_AF, Ice_QExt, Ice_SSA, Ice_AF

    ! Chou
    if ( ( 0.0d2          <= 1.0/Wavelength ) .and. &
      &  ( 1.0/Wavelength <= 340.0d2        ) ) then
      !  1
      Water_QExt = 0.08641+0.01769*(PartSize*1e6)-1.5572e-3*(PartSize*1e6)**2+3.4896e-5*(PartSize*1e6)**3
      Ice_QExt   = (-0.44171+0.62951/(PartSize*1e6)**0.06465)
      Ice_SSA    = 0.17201+1.2229e-2*(PartSize*1e6)-1.4837e-4*(PartSize*1e6)**2+5.8020e-7*(PartSize*1e6)**3
      Ice_AF     = 0.57867+1.0135e-2*(PartSize*1e6)-1.1142e-4*(PartSize*1e6)**2+4.1537e-7*(PartSize*1e6)**3
    else if ( ( 340.0d2        <= 1.0/Wavelength ) .and. &
      &       ( 1.0/Wavelength <= 540.0d2        ) ) then
      !  2
      Water_QExt = 0.22027+0.00997*(PartSize*1e6)-1.8719e-3*(PartSize*1e6)**2+5.3112e-5*(PartSize*1e6)**3
      Ice_QExt   = (-0.13727+0.61291/(PartSize*1e6)**0.28962)
      Ice_SSA    = 0.81470-2.7293e-3*(PartSize*1e6)+9.7816e-8*(PartSize*1e6)**2+5.7650e-8*(PartSize*1e6)**3
      Ice_AF     = 0.72259+3.1149e-3*(PartSize*1e6)-1.9927e-5*(PartSize*1e6)**2+5.6024e-8*(PartSize*1e6)**3
    else if ( ( 540.0d2        <= 1.0/Wavelength ) .and. &
      &       ( 1.0/Wavelength <= 800.0d2        ) ) then
      !  3
      Water_QExt = 0.38074-0.03027*(PartSize*1e6)+1.0154e-3*(PartSize*1e6)**2-1.1849e-5*(PartSize*1e6)**3
      Ice_QExt   = (-0.01878+1.67680/(PartSize*1e6)**0.79080)
      Ice_SSA    = 0.54859-4.8273e-4*(PartSize*1e6)+5.4353e-6*(PartSize*1e6)**2-1.5679e-8*(PartSize*1e6)**3
      Ice_AF     = 0.76109+4.5449e-3*(PartSize*1e6)-4.6199e-5*(PartSize*1e6)**2+1.6446e-7*(PartSize*1e6)**3
    else if ( ( 800.0d2        <= 1.0/Wavelength ) .and. &
      &       ( 1.0/Wavelength <= 980.0d2        ) ) then
      !  4
      Water_QExt = 0.15587-0.00371*(PartSize*1e6)-7.7705e-4*(PartSize*1e6)**2+2.0547e-5*(PartSize*1e6)**3
      Ice_QExt   = (-0.01896+1.06510/(PartSize*1e6)**0.69493)
      Ice_SSA    = 0.39218+4.1717e-3*(PartSize*1e6)-4.8869e-5*(PartSize*1e6)**2+1.9144e-7*(PartSize*1e6)**3
      Ice_AF     = 0.86934+2.7474e-3*(PartSize*1e6)-3.1301e-5*(PartSize*1e6)**2+1.1959e-7*(PartSize*1e6)**3
    else if ( ( 980.0d2        <= 1.0/Wavelength ) .and. &
      &       ( 1.0/Wavelength <= 1100.0d2       ) ) then
      !  5
      Water_QExt = 0.05518+0.04544*(PartSize*1e6)-4.2067e-3*(PartSize*1e6)**2+1.0184e-4*(PartSize*1e6)**3
      Ice_QExt   = (-0.04788+0.88178/(PartSize*1e6)**0.54492)
      Ice_SSA    = 0.71773-3.3640e-3*(PartSize*1e6)+1.9713e-5*(PartSize*1e6)**2-3.3189e-8*(PartSize*1e6)**3
      Ice_AF     = 0.89103+1.8513e-3*(PartSize*1e6)-1.6551e-5*(PartSize*1e6)**2+5.5193e-8*(PartSize*1e6)**3
    else if ( ( 1100.0d2       <= 1.0/Wavelength ) .and. &
      &       ( 1.0/Wavelength <= 1215.0d2       ) ) then
      !  6
      Water_QExt = 0.12724+0.04751*(PartSize*1e6)-5.2037e-3*(PartSize*1e6)**2+1.3711e-4*(PartSize*1e6)**3
      Ice_QExt   = (-0.02265+1.57390/(PartSize*1e6)**0.76161)
      Ice_SSA    = 0.77345-5.5228e-3*(PartSize*1e6)+4.8379e-5*(PartSize*1e6)**2-1.5151e-7*(PartSize*1e6)**3
      Ice_AF     = 0.86325+2.1408e-3*(PartSize*1e6)-1.6846e-5*(PartSize*1e6)**2+4.9473e-8*(PartSize*1e6)**3
    else if ( ( 1215.0d2       <= 1.0/Wavelength ) .and. &
      &       ( 1.0/Wavelength <= 1380.0d2       ) ) then
      !  7
      Water_QExt = 0.30390+0.01656*(PartSize*1e6)-3.5271e-3*(PartSize*1e6)**2+1.0828e-4*(PartSize*1e6)**3
      Ice_QExt   = (-0.01038+2.15640/(PartSize*1e6)**0.89045)
      Ice_SSA    = 0.74975-5.6604e-3*(PartSize*1e6)+5.6475e-5*(PartSize*1e6)**2-1.9664e-7*(PartSize*1e6)**3
      Ice_AF     = 0.85064+2.5028e-3*(PartSize*1e6)-2.0812e-5*(PartSize*1e6)**2+6.3427e-8*(PartSize*1e6)**3
    else if ( ( 1380.0d2       <= 1.0/Wavelength ) .and. &
      &       ( 1.0/Wavelength <= 1900.0d2       ) ) then
      !  8
      Water_QExt = 0.63617-0.06287*(PartSize*1e6)+2.2350e-3*(PartSize*1e6)**2-2.3177e-5*(PartSize*1e6)**3
      Ice_QExt   = (-0.00450+2.51370/(PartSize*1e6)**0.95989)
      Ice_SSA    = 0.69011-4.5348e-3*(PartSize*1e6)+4.9322e-5*(PartSize*1e6)**2-1.8255e-7*(PartSize*1e6)**3
      Ice_AF     = 0.86945+2.4615e-3*(PartSize*1e6)-2.3882e-5*(PartSize*1e6)**2+8.2431e-8*(PartSize*1e6)**3
    else if ( ( 1900.0d2       <= 1.0/Wavelength ) .and. &
      &       ( 1.0/Wavelength <= 3000.0d2       ) ) then
      !  9
      Water_QExt = 1.15470-0.19282*(PartSize*1e6)+1.2084e-2*(PartSize*1e6)**2-2.5612e-4*(PartSize*1e6)**3
      Ice_QExt   = (-0.00044+3.15050/(PartSize*1e6)**1.03750)
      Ice_SSA    = 0.83963-6.7253e-3*(PartSize*1e6)+6.1900e-5*(PartSize*1e6)**2-2.0862e-7*(PartSize*1e6)**3
      Ice_AF     = 0.80122+3.1906e-3*(PartSize*1e6)-2.4856e-5*(PartSize*1e6)**2+7.2411e-8*(PartSize*1e6)**3
    else
      Water_QExt = 0.0
      Ice_QExt   = (3.33e-4+2.52/(PartSize*1e6))
      Ice_SSA    = 0.0
      Ice_AF     = 0.0
    end if
    Water_QExt = Water_QExt *4.0/3*RhoWater*PartSize*1e3
    Ice_QExt   = Ice_QExt   *4.0/3*RhoIce  *PartSize*1e3

    write( 60, * ) Wavelength, Water_QExt, Water_SSA, Water_AF, Ice_QExt, Ice_SSA, Ice_AF
  end do



  call H2OWaterFinalize
  call H2OIceFinalize

end program main
