program main

  use vtype_module
  use constants0
  use fi_module
  use ni3_module
  use io_wnlist, only : &
    & IOWNListInqDimLen, &
    & IOWNListRead

  use wn_utils, only : &
    & WNUtilsCalcDelWaveNum

  use planck_func, only : PF

  implicit none


  character(extstr) :: CtlFN1 = 'calc_str_spe.conf'
  character(extstr) :: CtlFN2 = 'calc_str_spe_file.conf'

  integer           :: iostat

  character(len=extstr)          :: OutFileName


  integer                        :: InNWaveNum
  real(dp), allocatable          :: w_InWaveNum(:)
  real(dp), allocatable          :: w_InSpe    (:)

  integer                        :: OutNWaveNum
  real(dp), allocatable          :: w_OutWaveNum   (:)
  real(dp), allocatable          :: w_OutSpe       (:)
  real(dp), allocatable          :: w_DelOutWaveNum(:)

  real(dp), allocatable          :: a_Array(:)

  real(dp)                           :: StrTemp
  real(dp)                           :: StrFluxTOA

  real(dp)                           :: Sum


  integer(i4b)              :: FileUnit

  integer                            :: NDims
  character(len=extstr), allocatable :: a_DimNames(:)

  integer                   :: InNcID
  integer                   :: OutNcID
  character(len=extstr)     :: Mode
  character(len=extstr)     :: Name
  character(len=extstr)     :: StdName
  character(len=extstr)     :: LongName
  character(len=extstr)     :: Units
  character(len=extstr)     :: Comment

  logical              , parameter :: FlagNC = .true.
  character(len=extstr)            :: WaveNumFileName

  logical                   :: FlagInFileASCII
  character(len=extstr)     :: InFileName
  character(len=extstr)     :: WaveNumName
  character(len=extstr)     :: SpeName


  integer :: iWaveNum
  integer :: l


  namelist /spectrum_nml/ &
    & StrTemp, StrFluxTOA
  namelist /file_nml/ &
    & WaveNumFileName, &
    & InFileName, &
    & FlagInFileASCII, &
    & WaveNumName, SpeName, &
    & OutFileName, &
    & Comment


  StrTemp = 6000.0d0
  StrTemp =   -1.0d0

  StrFluxTOA = 1367.0d0

  !======================
  WaveNumFileName  = "./data.nc.nc"
  InFileName       = "./data.nc.nc"
  FlagInFileASCII  = .false.
  OutFileName      = "./out/ac.nc"
  WaveNumName      = "WaveNum"
  SpeName          = "Spe"
  !======================
  Comment          = ''


  ! Read a namelist file

  call fi_open( CtlFN1, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = spectrum_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in spectrum_nml is something wrong.'
  end if
  write( 6, nml = spectrum_nml )
  !
  close( FileUnit )

  call fi_open( CtlFN2, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = file_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in file_nml is something wrong.'
  end if
  write( 6, nml = file_nml )
  !
  close( FileUnit )



  ! read wavenumber
  call IOWNListInqDimLen(      &
    & FlagNC, WaveNumFileName, &
    & OutNWaveNum              &
    & )
  allocate( w_OutWaveNum( OutNWaveNum ) )
  call IOWNListRead(                        &
    & FlagNC, WaveNumFileName, OutNWaveNum, &
    & w_OutWaveNum                          &
    & )


  allocate( w_OutSpe    ( OutNWaveNum ) )


  if ( StrTemp >= 0.0d0 ) then

    do iWaveNum = 1, OutNWaveNum
      w_OutSpe(iWaveNum) = PI * PF( w_OutWaveNum(iWaveNum), StrTemp )
    end do
    if ( StrFluxTOA > 0.0d0 ) then
      w_OutSpe = w_OutSpe * StrFluxTOA / ( StB * StrTemp**4 )
    else
      w_OutSpe = w_OutSpe
    end if

  else

    if ( FlagInFileASCII ) then
      Mode = "read"
      InNWaveNum = fi_lc( InFileName )
      allocate( w_InWaveNum( InNWaveNum ) )
      allocate( w_InSpe    ( InNWaveNum ) )
      call fi_open( InFileName, Mode, FileUnit )
      do l = 1, InNWaveNum
        read( FileUnit, * ) w_InWaveNum(l), w_InSpe(l)
        write( 6, * ) w_InWaveNum(l), w_InSpe(l)
      end do
      close( FileUnit )
    else
      mode = "read"
      call ni3_open( InFileName, mode, InNcID )
      call ni3_inq_dimlen( InNcID, WaveNumName, InNWaveNum )
      allocate( w_InWaveNum( InNWaveNum ) )
      allocate( w_InSpe    ( InNWaveNum ) )
      call ni3_get_var( InNcID, WaveNumName, w_InWaveNum )
      call ni3_get_var( InNcID, SpeName    , w_InSpe     )
      call ni3_close( InNcID )
    end if

    if ( w_InWaveNum(1) > w_InWaveNum(2) ) then
      allocate( a_Array( InNWaveNum ) )
      do l = 1, InNWaveNum
        a_Array(l) = w_InWaveNum(InNWaveNum-l+1)
      end do
      w_InWaveNum = a_Array
      do l = 1, InNWaveNum
        a_Array(l) = w_InSpe(InNWaveNum-l+1)
      end do
      w_InSpe = a_Array
      deallocate( a_Array )
    end if


    do iWaveNum = 1, OutNWaveNum
      do l = 1+1, InNWaveNum
        if ( w_InWaveNum(l) >= w_OutWaveNum(iWaveNum) ) exit
      end do
      l = min( l, InNWaveNum )
      w_OutSpe(iWaveNum) = &
        &   ( w_InSpe(l)             - w_InSpe(l-1)     ) &
        & / ( w_InWaveNum(l)         - w_InWaveNum(l-1) ) &
        & * ( w_OutWaveNum(iWaveNum) - w_InWaveNum(l-1) ) &
        & + w_InSpe(l-1)
      w_OutSpe(iWaveNum) = max( w_OutSpe(iWaveNum), 0.0d0 )
    end do

    allocate( w_DelOutWaveNum( OutNWaveNum ) )
    call WNUtilsCalcDelWaveNum( &
      & OutNWaveNum, w_OutWaveNum,   &
      & w_DelOutWaveNum           &
      & )
    Sum = 0.0d0
    do iWaveNum = 1, OutNWaveNum
      Sum = Sum + w_OutSpe(iWaveNum) * w_DelOutWaveNum(iWaveNum)
    end do
    if ( StrFluxTOA > 0.0d0 ) then
      w_OutSpe = w_OutSpe * StrFluxTOA / Sum
    else
      w_OutSpe = w_OutSpe
    end if

  end if



  mode = "new"
  call ni3_open( OutFileName, mode, OutNcID )
  call ni3_put_att( OutNcID, '_global_', 'comment_strspe', Comment )
  Name     = "WaveNum"
  StdName  = Name
  LongName = Name
  Units    = "m-1"
  call ni3_set_dim( OutNcID, Name, NI3_DOUBLE, w_OutWaveNum, &
    & StdName, LongName, Units )
  Name     = "InFlux"
  StdName  = Name
  LongName = Name
  Units    = "W m-2 (m-1)-1"
  NDims = 1
  allocate( a_DimNames( NDims ) )
  a_DimNames( 1 ) = "WaveNum"
  call ni3_def_var( OutNcID, Name, NI3_DOUBLE, NDims, a_DimNames, &
    & StdName, LongName, Units )
  deallocate( a_DimNames )
  call ni3_put_var( OutNcID, Name, w_OutSpe )
  call ni3_close( OutNcID )


  write( 6, * ) 'Finish'

end program main

