#!/usr/bin/ruby
require "numru/ggraph"
include NumRu

pardir_data = "data_profile"
outncfilename = "flag.nc"
profncfilename = ""
profncfilename = "../prog01.1_mkprofile_Venus/out/Venus_VIRA+GasVMR+CldMMR+UVabsMMR_H2O_CO2_CO_SO2_HF_OCS_N2.nc"

a_files = []
Dir.glob( pardir_data+"/Profile_T-???_M??-001_M??-001_M??-001.nc" ) do |dir|
  a_files << dir
end
a_files.sort!

a_temp     = []
a_press    = []

t_temp     = []
pt_tblflag = []

for ifile in 0..(a_files.size-1)
  path = a_files[ifile]

  vname       = 'Temp'
  gp_temp = GPhys::NetCDF_IO.open( path, vname )
  na_temp = gp_temp.val

  vname       = 'FlagCalc'
  gp_flagcalc = GPhys::NetCDF_IO.open( path, vname )
  na_flagcalc = gp_flagcalc.val

  na_press = gp_flagcalc.coord('Press').val
  if na_press[-1] == 0 then
    na_press[-1] = na_press[-2] * na_press[-2] / na_press[-3]
    print "Zero pressure is modified to ", na_press[-1], ".\n"
  end

  a_press += na_press[ na_flagcalc.eq 1 ].to_a
  a_temp  += na_temp [ na_flagcalc.eq 1 ].to_a

  t_temp     << na_temp[0]
  pt_tblflag << na_flagcalc.to_a
end

if a_press.size > 0 then
  flag_plot = true
else
  flag_plot = false
end
if !flag_plot then
  print "STOP : There are not any points to be calculated.\n"
  exit
end

va_press =
  VArray.new( NArray.to_na( a_press ), 
              {"long_name"=>"Pressure", "units"=>"Pa"}, "Press" )
ax_press = Axis.new.set_pos(va_press)
#
va_temp =
  VArray.new( NArray.to_na( a_temp ), 
              {"long_name"=>"Temperature", "units"=>"K"}, "Temp" )
#
gp_temp = GPhys.new( Grid.new(ax_press), va_temp )


na_press = gp_flagcalc.coord('Press').val
if na_press[-1] == 0 then
  na_press[-1] = na_press[-2] * na_press[-2] / na_press[-3]
  print "Zero pressure is modified to ", na_press[-1], ".\n"
end
#
va_press =
  VArray.new( na_press, 
              {"long_name"=>"Pressure", "units"=>"Pa"}, "Press" )
ax_press = Axis.new.set_pos(va_press)
#
va_temp =
  VArray.new( NArray.to_na( t_temp ), 
              {"long_name"=>"Temperature", "units"=>"K"}, "Temp" )
ax_temp = Axis.new.set_pos(va_temp)
#
na_data = NArray.to_na( pt_tblflag )
va_data =
  VArray.new( na_data, 
              {"long_name"=>"Flag for calculation", "units"=>"1"}, "Flag" )
gp_flagcalc = GPhys.new( Grid.new(ax_press,ax_temp), va_data )


outncfile = NetCDF.create( outncfilename )
GPhys::NetCDF_IO.write( outncfile, gp_flagcalc )
outncfile.close


gp = gp_flagcalc


iws = (ARGV[0] || (puts ' WORKSTATION ID (I)  ? ;'; DCL::sgpwsn; gets)).to_i
DCL.gropn(iws)

#DCL.sldiv('y',4,2)
DCL.sgpset('isub', 96)     # control character of subscription: '_' --> '`'
DCL.sgpset('lfull',true)
DCL.uzfact(0.6)            # font size
DCL.sgpset('lclip', true)  # clip graph
DCL.glpset('lmiss',true)   # missing value

#DCL.sgscmn(4)  # blue-cyan-white-yellow-red
#DCL.sgscmn(5)  # gray-scale
#DCL.sgscmn(14) # blue-white-red
#levels = [110,120,130,140,150,160,170,180,190,200,210,220,230,240]
#patterns = [10999,15999,20999,25999,30999,35999,40999,50999,60999,65999,70999,75999,80999,90999,95999]

vpx1 = 0.1
vpx2 = 0.95
vpy1 = 0.1
vpy2 = 0.65

x1 = [gp.coord('Temp').val[0] - ( gp.coord('Temp').val[1] - gp.coord('Temp').val[0] ) / 2, 0.0].max
x2 = gp.coord('Temp').val[-1] + ( gp.coord('Temp').val[-1] - gp.coord('Temp').val[-2] ) / 2
y1 = gp.coord('Press').val[0] * gp.coord('Press').val[0] / gp.coord('Press').val[1]
y2 = gp.coord('Press').val[-1] * gp.coord('Press').val[-1] / gp.coord('Press').val[-2]
GGraph.set_fig 'itr'=> 2, 'viewport'=>[vpx1,vpx2,vpy1,vpy2],
               'window'=>[x1,x2,y1,y2]

#GGraph.tone gp, true, 'transpose'=>true, 'nlev'=>20, 'log'=>false  #, 'lev'=>levels, 'pat'=>patterns
##  GGraph.contour gp, false #, 'lev'=>levels
#GGraph.color_bar 'landscape'=>true, 'vlength'=>0.5

GGraph.mark gp_temp, true, 'title'=>"calculation points", 'exchange'=>true, 'type'=>1

if profncfilename.size > 0 then
  path = profncfilename
  vname      = 'Temp'
  gp_profile = GPhys::NetCDF_IO.open( path, vname )
  GGraph.line gp_profile, false, 'exchange'=>true, 'type'=>1
end

DCL.grcls
