module io_wnlist

  use vtype_module
  use ni3_module

  implicit none

  private

  public :: IOWNListInqDimLen
  public :: IOWNListRead
  public :: IOWNListOutput
  !---
  public :: IOWNListInqDimLenNC
  public :: IOWNListReadNC
  public :: IOWNListOutputNC
  !---
  public :: IOWNListInqDimLenASCII
  public :: IOWNListReadASCII
  public :: IOWNListOutputASCII

  !----------------------------------------------------------------------------

contains

  !----------------------------------------------------------------------------

  subroutine IOWNListInqDimLen( &
    & FlagNC, InFileName,       &
    & NWaveNum                  &
    & )

    logical     , intent(in ) :: FlagNC
    character(*), intent(in ) :: InFileName

    integer     , intent(out) :: NWaveNum


    !
    ! local variables
    !

    if ( FlagNC ) then
      call IOWNListInqDimLenNC( &
        & InFileName,                 &
        & NWaveNum                    &
        & )
    else
      call IOWNListInqDimLenASCII( &
        & InFileName,                    &
        & NWaveNum                       &
        & )
    end if


  end subroutine IOWNListInqDimLen

  !----------------------------------------------------------------------------

  subroutine IOWNListRead(          &
    & FlagNC, InFileName, NWaveNum, &
    & w_WaveNum                     &
    & )

    logical     , intent(in ) :: FlagNC
    character(*), intent(in ) :: InFileName
    integer     , intent(in ) :: NWaveNum
    real(DP)    , intent(out) :: w_WaveNum(1:NWaveNum)


    !
    ! local variables
    !


    if ( FlagNC ) then
      call IOWNListReadNC( &
        & InFileName, NWaveNum,  &
        & w_WaveNum              &
        & )
    else
      call IOWNListReadASCII( &
        & InFileName, NWaveNum,   &
        & w_WaveNum   &
        & )
    end if


  end subroutine IOWNListRead

  !----------------------------------------------------------------------------

  subroutine IOWNListOutput(                    &
    & FlagNC, OutFileName, NWaveNum, w_WaveNum  &
    & )

    logical     , intent(in ) :: FlagNC
    character(*), intent(in ) :: OutFileName
    integer     , intent(in ) :: NWaveNum
    real(dp)    , intent(in ) :: w_WaveNum(NWaveNum)


    ! Local variables
    !


    if ( FlagNC ) then
      call IOWNListOutputNC(      &
        & OutFileName, NWaveNum, w_WaveNum  &
        & )
    else
      call IOWNListOutputASCII(       &
        & OutFileName, NWaveNum, w_WaveNum  &
        & )
    end if


  end subroutine IOWNListOutput

  !----------------------------------------------------------------------------

  subroutine IOWNListInqDimLenNC( &
    & InFileName,                 &
    & NWaveNum                    &
    & )

    character(*), intent(in ) :: InFileName

    integer     , intent(out) :: NWaveNum


    !
    ! local variables
    !
    character(stdstr) :: DimName = 'WaveNum'
    character(stdstr) :: Mode
    integer           :: NcID


    Mode = "read"
    call ni3_open( InFileName, Mode, NcID )

    call ni3_inq_dimlen( NcID, DimName, NWaveNum )

    call ni3_close( NcID )


  end subroutine IOWNListInqDimLenNC

  !----------------------------------------------------------------------------

  subroutine IOWNListReadNC( &
    & InFileName, NWaveNum,  &
    & w_WaveNum              &
    & )

    character(*), intent(in ) :: InFileName
    integer     , intent(in ) :: NWaveNum
    real(DP)    , intent(out) :: w_WaveNum(1:NWaveNum)


    !
    ! local variables
    !
    character(stdstr) :: DimName = 'WaveNum'
    character(stdstr) :: Mode
    integer           :: NcID


    Mode = "read"
    call ni3_open( InFileName, Mode, NcID )

    call ni3_get_var( NcID, DimName, w_WaveNum )

    call ni3_close( NcID )


  end subroutine IOWNListReadNC

  !----------------------------------------------------------------------------

  subroutine IOWNListOutputNC(      &
    & OutNcFn, NWaveNum, w_WaveNum  &
    & )

    character(*), intent(in) :: OutNcFn
    integer     , intent(in) :: NWaveNum
    real(dp)    , intent(in) :: w_WaveNum(NWaveNum)


    ! Local variables
    !
    integer                 :: NcID
    character(extstr)       :: Mode
    character(extstr)       :: Name
    character(extstr)       :: StdName
    character(extstr)       :: LongName
    character(extstr)       :: Units


    ! Output

    Mode = "new"
    call ni3_open( OutNcFN, Mode, NcID )

    Name     = "WaveNum"
    StdName  = Name
    LongName = "wavenumber"
    Units    = "m-1"
    call ni3_set_dim( NcID, Name, NI3_DOUBLE, w_WaveNum, &
      & StdName, LongName, Units )

    call ni3_close( NcID )


  end subroutine IOWNListOutputNC

  !----------------------------------------------------------------------------

  subroutine IOWNListInqDimLenASCII( &
    & InFileName,                    &
    & NWaveNum                       &
    & )

    use fi_module, only : fi_lc

    character(*), intent(in ) :: InFileName
    integer     , intent(out) :: NWaveNum


    !
    ! local variables
    !


    NWaveNum = fi_lc( InFileName )


  end subroutine IOWNListInqDimLenASCII

  !----------------------------------------------------------------------------

  subroutine IOWNListReadASCII( &
    & InFileName, &
    & NWaveNum,   &
    & w_WaveNum   &
    & )

    use fi_module, only : fi_open

    character(*), intent(in ) :: InFileName
    integer     , intent(in ) :: NWaveNum
    real(DP)    , intent(out) :: w_WaveNum(NWaveNum)


    ! Local variables
    !
    integer :: FileUnit

    integer :: iWaveNum


    !
    ! Input
    !

    call fi_open( InFileName, "read", FileUnit )
    do iWaveNum = 1, NWaveNum
      read( FileUnit, * ) w_WaveNum(iWaveNum)
    end do
    close( FileUnit )


  end subroutine IOWNListReadASCII

  !----------------------------------------------------------------------------

  subroutine IOWNListOutputASCII(       &
    & OutFileName, NWaveNum, w_WaveNum  &
    & )

    use fi_module, only : fi_open

    character(*), intent(in) :: OutFileName
    integer     , intent(in) :: NWaveNum
    real(DP)    , intent(in) :: w_WaveNum(NWaveNum)


    ! Local variables
    !
    integer :: FileUnit

    integer :: iWaveNum


    !
    ! Output
    !

    call fi_open( OutFileName, "write", FileUnit )
    do iWaveNum = 1, NWaveNum
      write( FileUnit, * ) w_WaveNum(iWaveNum)
    end do
    close( FileUnit )


  end subroutine IOWNListOutputASCII

  !----------------------------------------------------------------------------

end module io_wnlist
